-- Feature_cmds.lua
-- Feature Control Routines

--====================================================================
-- this script controls some of the features of the Resurgence maps
-- implementing the clickable buttons which make various components visible
--====================================================================
-- ----------------------------
-- define tables of visibility states
-- ----------------------------

-- table of all affected objects
-- this is used to hide them all before drawing the currently visible ones

ft_all = {
"ToOpenControlButton",
"ToOpen2ControlButton",
"ToCloseControlButton",
"ResurgenceClosed",
"ResurgenceDim",
"ToDimControlButton",
"ToBrightControlButton",
"ResurgenceCommOffline",
"ResurgenceReady",
"FeatureBackdrop",
"CommButton",
"Comm2Button",
"Comm3Button",
"Comm4Button",
"CommInfo",
"CommFail",
"CommInfoSatellite",
"CommInfoWaves",
"HexControlButton",
"Hex2ControlButton",
"HexInfo",
-- comm interaction
"CommPwrOffButtonFilled",
"CommPwrOnButtonFilled",
"LoginOffButtonFilled",
"LoginOnButtonFilled",
"EncrOffButtonFilled",
"EncrLowButtonFilled",
"EncrMedButtonFilled",
"EncrHighButtonFilled",
"CommPwrOffButton",
"CommPwrOnButton",
"LoginOffButton",
"LoginOnButton",
"LoginOnButtonDim",
"EncrOffButton",
"EncrLowButton",
"EncrMedButton",
"EncrHighButton",
"EncrLowButtonDim",
"EncrMedButtonDim",
"EncrHighButtonDim",
-- initialization
"ResurgenceInitializing",
"ResurgenceTestOK1",
"ResurgenceTestOK2",
"ResurgenceTestOK3",
"ResurgenceProgressOK1",
"ResurgenceProgressOK2",
"ResurgenceProgressFail3",
}
ft_openit = {
"ToOpen2ControlButton",
"ToDimControlButton",
"Comm2Button",
"Hex2ControlButton",
"ResurgenceInitializing",
"ResurgenceTestOK1",
}
ft_openit_02 = {
"ToCloseControlButton",
"Comm2Button",
"ResurgenceCommOffline",
}
ft_closeit = {
"ToOpenControlButton",
"ResurgenceClosed",
}
ft_brightenit = {
"ToCloseControlButton",
"ToDimControlButton",
"CommButton",
"Hex2ControlButton",
}
ft_dimit = {
"ToCloseControlButton",
"ToBrightControlButton",
"CommButton",
"Hex2ControlButton",
"ResurgenceDim",
}
ft_comm = {
"CommInfoSatellite",
"CommInfoWaves",
"Comm4Button",
"CommInfo",
"ToCloseControlButton",
"ToDimControlButton",
"HexControlButton",
"CommPwrOffButton",
"CommPwrOnButtonFilled",
"LoginOffButton",
"LoginOnButtonFilled",
"EncrOffButton",
"EncrLowButton",
"EncrMedButton",
"EncrHighButtonFilled",
}
ft_comm2 = {
"Comm3Button",
"CommFail",
"ToCloseControlButton",
--
"CommPwrOffButtonFilled",
"CommPwrOnButton",
"LoginOffButtonFilled",
"LoginOnButtonDim",
"EncrOffButtonFilled",
"EncrLowButtonDim",
"EncrMedButtonDim",
"EncrHighButtonDim",
}
ft_comm3 = {
"CommInfoSatellite",
"CommInfoWaves",
"Comm4Button",
"CommInfo",
"ToCloseControlButton",
"CommPwrOffButton",
"CommPwrOnButtonFilled",
"LoginOffButton",
"LoginOnButtonFilled",
"EncrOffButton",
"EncrLowButton",
"EncrMedButton",
"EncrHighButtonFilled",
}
ft_comm4 = {
"CommButton",
"ToCloseControlButton",
"ToDimControlButton",
"Hex2ControlButton",
"FeatureBackdrop",
--"ResurgenceReady",
}
ft_hex = {
-- let background shine through
"Hex2ControlButton",
-- "HexInfo",
"CommButton",
"ToCloseControlButton",
"ToDimControlButton",
}
ft_hex2 = {
"HexControlButton",
"CommButton",
"ToCloseControlButton",
"ToDimControlButton",
"ResurgenceReady",
}

-- ----------------------------
-- functions to hide and show the objects
-- ----------------------------

-- ---------------
fc_show = function (name)
   celestia:find(name):setvisible(true)
end

-- ---------------
fc_hide = function (name)
   celestia:find(name):setvisible(false)
end
-- ---------------

fc_show_tbl = function (map_table)
  for j,r in ipairs(map_table) do
    fc_show(map_table[j])
  end -- for j,r loop
end
-- ---------------
fc_hide_tbl = function (map_table)
  for j,r in ipairs(map_table) do
   celestia:find(map_table[j]):setvisible(false)
  end -- for j,r loop
end

fc_show_map = function (map_table)
  fc_hide_tbl(ft_all)
  fc_show_tbl (map_table)
end

fc_hide_others = function ()
-- hides maps of other resurgence addons

   local obj, ctype

   obj=celestia:find("ResurgenceCore/Maps")
   ctype = obj:type()
   if (ctype == "component")
   then
     fc_hide_tbl (rmt_all) -- resurgence addon
   end

   obj=celestia:find("ResurgenceCore/Maps_M")
   ctype = obj:type()
   if (ctype == "component")
   then
     fc_hide_tbl (mt_all) -- mundilfoeri addon
   end

   obj=celestia:find("ResurgenceCore/Maps_SN")
   ctype = obj:type()
   if (ctype == "component")
   then
     fc_hide_tbl (snt_all) -- surya/nyir
   end

   obj=celestia:find("ResurgenceCore/Maps_S")
   ctype = obj:type()
   if (ctype == "component")
   then
    fc_hide_tbl (st_all) -- sola
   end
end
--====================================================================
-- command routines
--====================================================================

fc_initialize = function ()
  fc_show_map(ft_openit)
  fv_iT0 = 86400.0*celestia:gettime()
end

fv_1st = true
fv_2nd = false

fc_init = function ()
  if (fv_1st)
  then
    fv_1st = false
    fv_2nd = true
    return
  elseif (fv_2nd)
  then
    fv_2nd = false

-- initialize comm waves
    fv_wT0 = 86400.0*celestia:gettime()
    fv_wR2 = celestia:find("CommInfoWaves"):radius()
    fv_wR1 = fv_wR2/2.0

-- everything is turned off (hidden)
    fc_show_map(ft_closeit)
  end
end

fc_test_for_orrery = function()
-- multiple orreries cause celestia to crash :(
  local obj
    obj =celestia:find("GotoSysButton")
    if (obj:visible()) then obj:setvisible(false) ; return end

    obj =celestia:find("ShowMOrreryButton")
    if (obj:visible()) then obj:setvisible(false) ; return  end

    obj =celestia:find("ShowSOrreryButton")
    if (obj:visible())    then obj:setvisible(false) ; return end

    obj =celestia:find("ShowSNOrreryButton")
    if (obj:visible())    then obj:setvisible(false) ; return end
end

ft_neosol = {
"GalaxyButton",
"GalaxyMapButton3",
"NeosolButton",
"NSOButton",
"NSSuryaButton",
"NSNayirButton",
"NSMundilfoeriButton",
"NSSolaButton",
"NSHyperionButton",
}
-- hide spurious buttons on initial map
fc_test_for_buttons = function()
    local obj =celestia:find("GalaxyMap_nolbl")
    if (obj:visible()) 
    then
      fc_hide_tbl(ft_neosol)
    end
end
--===============
-- communications control
--===============
ft_pwroffhide = {
"CommPwrOffButton",
"CommPwrOnButtonFilled",
"LoginOffButton",
"LoginOnButton",
"LoginOnButtonFilled",
"EncrOffButton",
"EncrLowButtonFilled",
"EncrMedButtonFilled",
"EncrHighButtonFilled",
"EncrLowButton",
"EncrMedButton",
"EncrHighButton",
"CommInfo",
"CommInfoSatellite",
"CommInfoWaves",
}
ft_pwroffshow = {
"CommPwrOffButtonFilled",
"CommPwrOnButton",
"LoginOffButtonFilled",
"LoginOnButtonDim",
"EncrOffButtonFilled",
"EncrLowButtonDim",
"EncrMedButtonDim",
"EncrHighButtonDim",
"CommFail",
}
ft_pwronhide = {
"CommPwrOffButtonFilled",
"CommPwrOnButton",
"LoginOnButtonDim",
"EncrLowButtonDim",
"EncrMedButtonDim",
"EncrHighButtonDim",
"LoginOffButton",
"LoginOnButton",
"LoginOnButtonFilled",
}
ft_pwronshow = {
"CommPwrOffButton",
"CommPwrOnButtonFilled",
"CommInfoSatellite",
"EncrLowButton",
"EncrMedButton",
"EncrHighButton",
"LoginOffButtonFilled",
"LoginOnButtonDim",
}
ft_loginoffhide = {
"LoginOffButton",
"LoginOnButtonFilled",
"CommInfo",
}
ft_loginoffshow = {
"LoginOffButtonFilled",
"LoginOnButton",
"CommInfoSatellite",
"CommInfoWaves",
"CommFail",
}
ft_loginonhide = {
"LoginOnButton",
"LoginOffButtonFilled",
"CommFail",
}
ft_loginonshow = {
"LoginOnButtonFilled",
"LoginOffButton",
"CommInfo",
"CommInfoSatellite",
"CommInfoWaves",
}
ft_encroffhide = {
"EncrOffButton",
"EncrLowButtonFilled",
"EncrMedButtonFilled",
"EncrHighButtonFilled",
"LoginOnButtonFilled",
"LoginOnButton",
"LoginOffButton",
"CommInfoWaves",
"CommInfo",
}
ft_encroffshow = {
"EncrOffButtonFilled",
"EncrLowButton",
"EncrMedButton",
"EncrHighButton",
"LoginOnButtonDim",
"LoginOffButtonFilled",
"CommInfoSatellite",
"CommFail",
}
ft_encrlowhide = {
"EncrOffButtonFilled",
"EncrLowButton",
"EncrMedButtonFilled",
"EncrHighButtonFilled",
"LoginOnButtonDim",
}
ft_encrlowshow = {
"EncrOffButton",
"EncrLowButtonFilled",
"EncrMedButton",
"EncrHighButton",
"LoginOnButton",
"CommInfoWaves",
"CommInfoSatellite",
}
ft_encrmedhide = {
"EncrOffButtonFilled",
"EncrLowButtonFilled",
"EncrMedButton",
"EncrHighButtonFilled",
"LoginOnButtonDim",
}
ft_encrmedshow = {
"EncrOffButton",
"EncrLowButton",
"EncrMedButtonFilled",
"EncrHighButton",
"LoginOnButton",
"CommInfoWaves",
"CommInfoSatellite",
}
ft_encrhighhide = {
"EncrOffButtonFilled",
"EncrLowButtonFilled",
"EncrMedButtonFilled",
"EncrHighButton",
"LoginOnButtonDim",
}
ft_encrhighshow = {
"EncrOffButton",
"EncrLowButton",
"EncrMedButton",
"EncrHighButtonFilled",
"LoginOnButton",
"CommInfoWaves",
"CommInfoSatellite",
}
--=====
-- ------- progress bar -------------

fc_cycle_init = function ()
  if ( celestia:find("ResurgenceTestOK1"):visible()) then
    local fv_iTd = 86400.0*celestia:gettime()
    local dt = fv_iTd -fv_iT0
    if (dt > 1.5)
    then
      fv_iT0 = fv_iTd
      celestia:find("ResurgenceTestOK1"):setvisible(false)
      celestia:find("ResurgenceProgressOK1"):setvisible(true)
      celestia:find("ResurgenceTestOK2"):setvisible(true)
    end
  elseif ( celestia:find("ResurgenceTestOK2"):visible()) then
    local fv_iTd = 86400.0*celestia:gettime()
    local dt = fv_iTd -fv_iT0
    if (dt > 1.5)
    then
      fv_iT0 = fv_iTd
      celestia:find("ResurgenceTestOK2"):setvisible(false)
      celestia:find("ResurgenceProgressOK2"):setvisible(true)
      celestia:find("ResurgenceTestOK3"):setvisible(true)
    end
  elseif ( celestia:find("ResurgenceTestOK3"):visible()) then
    local fv_iTd = 86400.0*celestia:gettime()
    local dt = fv_iTd -fv_iT0
    if (dt > 1.5)
    then
      fv_iT0 = fv_iTd
      celestia:find("ResurgenceTestOK3"):setvisible(false)
      celestia:find("ResurgenceProgressFail3"):setvisible(true)
    end
  elseif ( celestia:find("ResurgenceProgressFail3"):visible()) then
    local fv_iTd = 86400.0*celestia:gettime()
    local dt = fv_iTd -fv_iT0
    if (dt > 1.5)
    then
      fv_iT0 = fv_iTd
      fc_show_map(ft_openit_02)
    end
  end

end
-- -------comm radiation --------
fc_cycle_waves = function ()
  local obj = celestia:find("CommInfoWaves")
  if ( obj:visible()) then
    local fv_wTd = 86400.0*celestia:gettime()
    local dt = fv_wTd -fv_wT0
    if (dt > 1.0 or dt < 0.0)
    then
      dt = 0.0
      fv_wT0 = fv_wTd
    end
    obj:setradius(fv_wR1 + (fv_wR2-fv_wR1)*dt)
  end
end
-- --------- comm buttons --------
fc_pwroff = function ()
  fc_hide_tbl(ft_pwroffhide)
  fc_show_tbl(ft_pwroffshow)
end
fc_pwron = function ()
    fc_hide_tbl(ft_pwronhide)
    fc_show_tbl(ft_pwronshow)
end

fc_loginoff = function ()
  fc_hide_tbl(ft_loginoffhide)
  fc_show_tbl(ft_loginoffshow)
end

fc_loginon = function ()
    fc_show_map(ft_comm3)
end
fc_encroff = function ()
  fc_hide_tbl(ft_encroffhide)
  fc_show_tbl(ft_encroffshow)
end
fc_encrlow = function ()
  if ( celestia:find("CommPwrOnButtonFilled"):visible()) then
    fc_hide_tbl(ft_encrlowhide)
    fc_show_tbl(ft_encrlowshow)
  end
end
fc_encrmed = function ()
  if ( celestia:find("CommPwrOnButtonFilled"):visible()) then
    fc_hide_tbl(ft_encrmedhide)
    fc_show_tbl(ft_encrmedshow)
  end
end
fc_encrhigh = function ()
  if ( celestia:find("CommPwrOnButtonFilled"):visible()) then
    fc_hide_tbl(ft_encrhighhide)
    fc_show_tbl(ft_encrhighshow)
  end
end


--====================================================================
-- command processor table and routines:
-- the name of the selected object is interpreted as a command
--====================================================================

-- fv_selP = previous selection
-- to be restored immediately after recognizing a command
-- assume initial selection (if any) when Celestia starts isn't a command

-- must be initialized
-- but can't use getselection during startup of Celestia
-- which is when this code is initially executed
-- because nothing has been selected yet

fv_selP = nil

-- ======================================= 
-- table of commands:
-- command text (i.e.name of selected object) = { command function, arg_of_fn },

ft_cmds = {
 ToCloseControlButton = {fc_show_map,ft_closeit},
 ToOpenControlButton = {fc_initialize,nil},
-- ToOpenControlButton = {fc_show_map,ft_openit},
 ToOpen2ControlButton = {fc_show_map,ft_open2},
 ToDimControlButton = {fc_show_map,ft_dimit},
 ToBrightControlButton = {fc_show_map,ft_brightenit},
 CommButton = {fc_show_map,ft_comm},
 Comm2Button = {fc_show_map,ft_comm2},
 Comm3Button = {fc_show_map,ft_comm3},
 Comm4Button = {fc_show_map,ft_comm4},
 HexControlButton = {fc_show_map,ft_hex},
 Hex2ControlButton = {fc_show_map,ft_hex2},
 ResurgenceInitializing = {fc_show_map,ft_openit_02},
 ToOpen2ControlButton = {fc_show_map,ft_openit_02},
-- comm interaction
CommPwrOffButton = { fc_pwroff,nil},
CommPwrOnButton = { fc_pwron,nil},
LoginOffButton = { fc_loginoff,nil},
LoginOnButton = { fc_loginon,nil},
EncrOffButton = { fc_encroff,nil},
EncrLowButton = { fc_encrlow,nil},
EncrMedButton = { fc_encrmed,nil},
EncrHighButton = { fc_encrhigh,nil},
}


-- ======================================= 
-- action function

fc_cmds = function (this_date)

fc_test_for_buttons()
fc_test_for_orrery()
fc_cycle_waves()
fc_cycle_init()

-- to implement commands only for the battery and its children
-- this code uses locals, fc_ routines and ft_ tables

  local sel = celestia:getselection()

  if (sel == fv_selP) then return 0 end
  
  local sname = sel:name()
  
-- get the two-element table (if any) matching the name of the selection

  local todo = ft_cmds[sname]
  
    if todo then 
     -- restore previous selection before executing this one
        if (fv_selP ~= nil) then celestia:select(fv_selP) end
	
	-- call the function [table element 1] associated with this command,
	-- passing it a single argument [table element 2]
	todo[1](todo[2])
	return 1
    end

-- This selection is not recognized as a command,
-- so save it for reversion after next command

	fv_selP = sel
	return 0
end
